@@ -20,6 +20,9 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import absolute_import, division, print_function, \
+    with_statement
+
 import sys
 import logging
 
@@ -49,19 +52,66 @@ def err(alg, key, iv, op, key_as_bytes=0, d=None, salt=None, i=1, padding=1):
 
 if has_m2:
     ciphers = {
-        'aes-128-cfb': (16, 16, create_cipher),
-        'aes-192-cfb': (24, 16, create_cipher),
-        'aes-256-cfb': (32, 16, create_cipher),
-        'bf-cfb': (16, 8, create_cipher),
-        'camellia-128-cfb': (16, 16, create_cipher),
-        'camellia-192-cfb': (24, 16, create_cipher),
-        'camellia-256-cfb': (32, 16, create_cipher),
-        'cast5-cfb': (16, 8, create_cipher),
-        'des-cfb': (8, 8, create_cipher),
-        'idea-cfb': (16, 8, create_cipher),
-        'rc2-cfb': (16, 8, create_cipher),
-        'rc4': (16, 0, create_cipher),
-        'seed-cfb': (16, 16, create_cipher),
+        b'aes-128-cfb': (16, 16, create_cipher),
+        b'aes-192-cfb': (24, 16, create_cipher),
+        b'aes-256-cfb': (32, 16, create_cipher),
+        b'bf-cfb': (16, 8, create_cipher),
+        b'camellia-128-cfb': (16, 16, create_cipher),
+        b'camellia-192-cfb': (24, 16, create_cipher),
+        b'camellia-256-cfb': (32, 16, create_cipher),
+        b'cast5-cfb': (16, 8, create_cipher),
+        b'des-cfb': (8, 8, create_cipher),
+        b'idea-cfb': (16, 8, create_cipher),
+        b'rc2-cfb': (16, 8, create_cipher),
+        b'rc4': (16, 0, create_cipher),
+        b'seed-cfb': (16, 16, create_cipher),
     }
 else:
     ciphers = {}
+
+
+def run_method(method):
+    from shadowsocks.crypto import util
+
+    cipher = create_cipher(method, b'k' * 32, b'i' * 16, 1)
+    decipher = create_cipher(method, b'k' * 32, b'i' * 16, 0)
+
+    util.run_cipher(cipher, decipher)
+
+
+def check_env():
+    # skip this test on pypy and Python 3
+    try:
+        import __pypy__
+        del __pypy__
+        from nose.plugins.skip import SkipTest
+        raise SkipTest
+    except ImportError:
+        pass
+    if bytes != str:
+        from nose.plugins.skip import SkipTest
+        raise SkipTest
+
+
+def test_aes_128_cfb():
+    check_env()
+    run_method(b'aes-128-cfb')
+
+
+def test_aes_256_cfb():
+    check_env()
+    run_method(b'aes-256-cfb')
+
+
+def test_bf_cfb():
+    check_env()
+    run_method(b'bf-cfb')
+
+
+def test_rc4():
+    check_env()
+    run_method(b'rc4')
+
+
+if __name__ == '__main__':
+    test_aes_128_cfb()
